@@ -1,6 +1,6 @@
 #ifndef _POLICY_POLICY_PURGE_CMD_H_
 #define _POLICY_POLICY_PURGE_CMD_H_
 
-struct cmd_func_block* policy_purge_funcblock(void);
+struct cmd_func_block policy_purge_funcblock;
 
 #endif /* _POLICY_POLICY_PURGE_CMD_H_ */
