@@ -1,6 +1,4 @@
 /*
- * $Id: cfg.h 5945 2011-11-30 11:54:30Z matthijs $
- *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,30 +24,27 @@
  *
  */
 
-/**
- * Signer engine configuration.
- *
- */
-
 #ifndef DAEMON_CONFIG_H
 #define DAEMON_CONFIG_H
 
 #include "config.h"
-#include "shared/allocator.h"
-#include "shared/locks.h"
-#include "shared/status.h"
-#include "wire/listener.h"
-
 #include <stdio.h>
 
+typedef struct engineconfig_struct engineconfig_type;
+
+#include "status.h"
+#include "hsm.h"
+#include "locks.h"
+#include "status.h"
+#include "wire/listener.h"
+
 /**
  * Engine configuration.
  *
  */
-typedef struct engineconfig_struct engineconfig_type;
 struct engineconfig_struct {
-    allocator_type* allocator;
     listener_type* interfaces;
+    hsm_repository_t* repositories;
     const char* cfg_filename;
     const char* zonelist_filename;
     const char* log_filename;
@@ -74,8 +69,8 @@ struct engineconfig_struct {
  * \return engineconfig_type* engine configuration
  *
  */
-engineconfig_type* engine_config(allocator_type* allocator,
-    const char* cfgfile, int cmdline_verbosity);
+engineconfig_type* 
+engine_config(const char* cfgfile, int cmdline_verbosity);
 
 /**
  * Check configuration.
