@@ -52,6 +52,8 @@ typedef struct zone_struct zone_type;
 #include "wire/buffer.h"
 #include "wire/notify.h"
 #include "wire/xfrd.h"
+#include "datastructure.h"
+#include "daemon/engine.h"
 
 struct schedule_struct;
 
@@ -79,16 +81,16 @@ struct zone_struct {
     /* zone transfers */
     xfrd_type* xfrd;
     notify_type* notify;
-    /* worker variables */
-    task_type* task; /* next assigned task */
     /* statistics */
     stats_type* stats;
-    lock_basic_type zone_lock;
-    lock_basic_type xfr_lock;
+    pthread_mutex_t zone_lock;
+    pthread_mutex_t xfr_lock;
     /* backing store for rrsigs (both domain as denial) */
     collection_class rrstore;
+    int zoneconfigvalid; /* flag indicating whether the signconf has at least once been read */
 };
 
+
 /**
  * Create a new zone.
  * \param[in] name zone name
@@ -127,7 +129,7 @@ ods_status zone_reschedule_task(zone_type* zone, schedule_type* taskq,
  * \return ods_status status
  *
  */
-ods_status zone_publish_dnskeys(zone_type* zone);
+ods_status zone_publish_dnskeys(zone_type* zone, int skip_hsm_access);
 
 /**
  * Unlink DNSKEY RRs.
@@ -235,13 +237,13 @@ void zone_cleanup(zone_type* zone);
  * \return ods_status status
  *
  */
-ods_status zone_backup2(zone_type* zone);
+ods_status zone_backup2(zone_type* zone, time_t nextResign);
 
 /**
  * Recover zone from backup.
  * \param[in] zone corresponding zone
  *
  */
-ods_status zone_recover2(zone_type* zone);
+ods_status zone_recover2(engine_type* engine, zone_type* zone);
 
 #endif /* SIGNER_ZONE_H */
